@@ -41,54 +41,9 @@
 #define gmtime_r(tp,tmp) (gmtime(tp)?(*(tmp)=*gmtime(tp),(tmp)):0)
 #endif
 
-/* Converts a string from Windows-UTF8 to classic-UTF8.
- * NOTE: If the returned value is non-NULL, the caller has to free the newly
- * allocated string using g_free()
- */
-gchar *
-utf8tolinux (const gchar *wstring)
-{
-	#ifndef HAVE_WINDOWS_TO_UTF8
-	/* newer Openchange (0.10+) doesn't provide windows_to_utf8 function,
-	   it does all the necessary decoding to utf8 transparently */
-	return g_strdup (wstring);
-	#else
-	TALLOC_CTX	*mem_ctx;
-	gchar		*newstr, *retval = NULL;
-	gint i;
-	gboolean all_ok = TRUE;
-
-	g_return_val_if_fail (wstring != NULL, NULL);
-
-	/* If all letters are with code <128 then there is nothing
-	   to be converted, thus return the original string copy.
-	   It's not working always, but should be fine for now.
-
-	   This is a temporary hack before the windows_to_utf8 will
-	   be fixed to not drop some letters.
-	*/
-	for (i = 0; all_ok && wstring[i]; i++) {
-		/* signed char, thus '> 128' means '< 0' */
-		all_ok = wstring[i] > 0;
-	}
-
-	if (all_ok)
-		return g_strdup (wstring);
-
-	mem_ctx = talloc_init ("ExchangeMAPI_utf8tolinux");
-
-	newstr = windows_to_utf8(mem_ctx, wstring);
-
-	if (g_utf8_validate (newstr, -1, NULL))
-		retval = g_strdup (newstr);
-	else
-		retval = g_strdup (wstring);
-
-	talloc_free (mem_ctx);
-
-	return retval;
-	#endif
-}
+/* Used for callout to krb5-auth-dialog */
+#define KRB_DBUS_PATH               "/org/gnome/KrbAuthDialog"
+#define KRB_DBUS_INTERFACE          "org.gnome.KrbAuthDialog"
 
 inline gchar *
 exchange_mapi_util_mapi_id_to_string (mapi_id_t id)
@@ -304,6 +259,35 @@ exchange_mapi_util_find_array_namedid (struct mapi_SPropValue_array *properties,
 	return res;
 }
 
+enum MAPISTATUS
+exchange_mapi_util_find_array_datetime_propval (struct timeval *tv, struct mapi_SPropValue_array *properties, uint32_t proptag)
+{
+	g_return_val_if_fail (tv != NULL, MAPI_E_INVALID_PARAMETER);
+	g_return_val_if_fail (properties != NULL, MAPI_E_INVALID_PARAMETER);
+
+	return get_mapi_SPropValue_array_date_timeval (tv, properties, proptag);
+}
+
+enum MAPISTATUS
+exchange_mapi_util_find_array_datetime_namedid (struct timeval *tv, struct mapi_SPropValue_array *properties, ExchangeMapiConnection *conn, mapi_id_t fid, uint32_t namedid)
+{
+	enum MAPISTATUS res = MAPI_E_NOT_FOUND;
+	uint32_t proptag;
+
+	g_return_val_if_fail (tv != NULL, MAPI_E_INVALID_PARAMETER);
+	g_return_val_if_fail (properties != NULL, MAPI_E_INVALID_PARAMETER);
+	g_return_val_if_fail (conn != NULL, MAPI_E_INVALID_PARAMETER);
+
+	proptag = exchange_mapi_connection_resolve_named_prop (conn, fid, namedid, NULL);
+	if (proptag != MAPI_E_RESERVED)
+		res = exchange_mapi_util_find_array_datetime_propval (tv, properties, proptag);
+
+	if (res == MAPI_E_NOT_FOUND)
+		res = exchange_mapi_util_find_array_datetime_propval (tv, properties, namedid);
+
+	return res;
+}
+
 ExchangeMAPIStream *
 exchange_mapi_util_find_stream (GSList *stream_list, uint32_t proptag)
 {
@@ -318,6 +302,27 @@ exchange_mapi_util_find_stream (GSList *stream_list, uint32_t proptag)
 	return NULL;
 }
 
+ExchangeMAPIStream *
+exchange_mapi_util_find_stream_namedid (GSList *stream_list, ExchangeMapiConnection *conn, mapi_id_t fid, uint32_t namedid)
+{
+	uint32_t proptag;
+	gconstpointer res = NULL;
+
+	g_return_val_if_fail (conn != NULL, NULL);
+
+	if (!stream_list)
+		return NULL;
+
+	proptag = exchange_mapi_connection_resolve_named_prop (conn, fid, namedid, NULL);
+	if (proptag != MAPI_E_RESERVED)
+		res = exchange_mapi_util_find_stream (stream_list, proptag);
+
+	if (!res)
+		res = exchange_mapi_util_find_stream (stream_list, namedid);
+
+	return (ExchangeMAPIStream *) res;
+}
+
 void
 exchange_mapi_util_free_attachment_list (GSList **attach_list)
 {
@@ -386,206 +391,6 @@ exchange_mapi_util_free_stream_list (GSList **stream_list)
 	*stream_list = NULL;
 }
 
-static void
-dump_bin (const uint8_t *bin, uint32_t bin_sz, const gchar *line_prefix)
-{
-	gint k, l, last;
-
-	if (!bin) {
-		g_print ("NULL");
-		return;
-	}
-
-	g_print ("%s", line_prefix);
-
-	last = 0;
-	for (k = 0; k < bin_sz; k++) {
-		if ((k > 0 && (k % 16) == 0)) {
-			g_print ("  ");
-			for (l = last; l < k; l++) {
-				uint8_t u8 = bin[l];
-
-				if ((l % 8) == 0)
-					g_print (" ");
-				if (u8 <= 32 || u8 >= 128)
-					g_print (".");
-				else
-					g_print ("%c", u8);
-			}
-
-			last = l;
-			g_print ("\n%s", line_prefix);
-		} else if (k > 0 && (k % 8) == 0) {
-			g_print ("  ");
-		}
-		g_print (" %02X", bin[k]);
-	}
-
-	if (last < k) {
-		l = k;
-
-		while ((l % 16) != 0) {
-			g_print ("   ");
-			if (l > 0 && (l % 8) == 0)
-				g_print ("  ");
-			l++;
-		}
-
-		g_print ("  ");
-		for (l = last; l < k; l++) {
-			uint8_t u8 = bin[l];
-
-			if ((l % 8) == 0)
-				g_print (" ");
-			if (u8 <= 32 || u8 >= 128)
-				g_print (".");
-			else
-				g_print ("%c", u8);
-		}
-	}
-}
-
-void
-exchange_mapi_debug_property_dump (struct mapi_SPropValue_array *properties)
-{
-	gint i = 0;
-
-	for (i = 0; i < properties->cValues; i++) {
-		for (i = 0; i < properties->cValues; i++) {
-			struct mapi_SPropValue *lpProp = &properties->lpProps[i];
-			const gchar *tmp =  get_proptag_name (lpProp->ulPropTag);
-			gchar t_str[26];
-			gint j = 0;
-			if (tmp && *tmp)
-				g_print("\n%s \t",tmp);
-			else
-				g_print("\n0x%08X \t", lpProp->ulPropTag);
-			switch (lpProp->ulPropTag & 0xFFFF) {
-			case PT_UNSPECIFIED:
-				g_print (" PT_UNSPECIFIED");
-				break;
-			case PT_NULL:
-				g_print (" PT_NULL");
-				break;
-			case PT_BOOLEAN:
-				g_print(" (bool) - %d", (bool) lpProp->value.b);
-				break;
-			case PT_I2:
-				g_print(" (uint16_t) - %d", lpProp->value.i);
-				break;
-			case PT_LONG:
-				g_print(" (long) - %u", lpProp->value.l);
-				break;
-			case PT_FLOAT:
-				g_print (" PT_FLOAT");
-				break;
-			case PT_DOUBLE:
-				g_print (" (double) -  %lf", (double)lpProp->value.dbl);
-				break;
-			case PT_CURRENCY:
-				g_print (" PT_CURRENCY");
-				break;
-			case PT_APPTIME:
-				g_print (" PT_APPTIME");
-			case PT_I8:
-				g_print (" (gint) - 0x%016" G_GINT64_MODIFIER "X", lpProp->value.d);
-				break;
-			case PT_SYSTIME: {
-					struct timeval t;
-					struct tm tm;
-					if (get_mapi_SPropValue_array_date_timeval (&t, properties, lpProp->ulPropTag) == MAPI_E_SUCCESS) {
-						gmtime_r (&(t.tv_sec), &tm);
-						strftime (t_str, 26, "%Y-%m-%dT%H:%M:%SZ", &tm);
-						g_print (" (struct FILETIME *) - %p\t (struct timeval) %s\t", &lpProp->value.ft, t_str);
-					}
-				}
-				break;
-			case PT_ERROR:
-				g_print (" (error) - "/* , lpProp->value.err */);
-				break;
-			case PT_STRING8:
-				g_print(" (string) - %s", lpProp->value.lpszA ? lpProp->value.lpszA : "null" );
-				break;
-			case PT_UNICODE:
-				if (lpProp)
-					g_print(" (unicodestring) - %s", lpProp->value.lpszW ? lpProp->value.lpszW : lpProp->value.lpszA ? lpProp->value.lpszA : "null");
-				break;
-			case PT_OBJECT:
-				g_print (" PT_OBJECT");
-				break;
-			case PT_CLSID:
-				g_print (" PT_CLSID");
-				break;
-			case PT_SVREID:
-				g_print (" PT_SVREID");
-				break;
-			case PT_SRESTRICT:
-				g_print (" PT_SRESTRICT");
-				break;
-			case PT_ACTIONS:
-				g_print (" PT_ACTIONS");
-				break;
-			case PT_BINARY:
-				g_print(" (struct SBinary_short *) - %p Binary data follows (size %d): \n", &lpProp->value.bin, lpProp->value.bin.cb);
-				dump_bin (lpProp->value.bin.lpb, lpProp->value.bin.cb, "     ");
-				break;
-			case PT_MV_STRING8:
-				g_print(" (struct mapi_SLPSTRArray *) (%d items)", lpProp->value.MVszA.cValues);
-				for (j = 0; j < lpProp->value.MVszA.cValues; j++) {
-					g_print ("\n   item[%d] = '%s'", j, lpProp->value.MVszA.strings[j].lppszA ? lpProp->value.MVszA.strings[j].lppszA : "[NULL]");
-				}
-				break;
-			case PT_MV_SHORT:
-				g_print (" PT_MV_SHORT");
-				break;
-			case PT_MV_LONG:
-				g_print (" PT_MV_LONG");
-				break;
-			case PT_MV_FLOAT:
-				g_print (" PT_MV_FLOAT");
-				break;
-			case PT_MV_DOUBLE:
-				g_print (" PT_MV_DOUBLE");
-				break;
-			case PT_MV_CURRENCY:
-				g_print (" PT_MV_CURRENCY");
-				break;
-			case PT_MV_APPTIME:
-				g_print (" PT_MV_APPTIME");
-				break;
-			case PT_MV_I8:
-				g_print (" PT_MV_I8");
-				break;
-			case PT_MV_UNICODE:
-				g_print (" PT_MV_UNICODE (%d items)", lpProp->value.MVszW.cValues);
-				for (j = 0; j < lpProp->value.MVszW.cValues; j++) {
-					g_print ("\n   item[%d] = '%s'", j, lpProp->value.MVszW.strings[j].lppszW ? lpProp->value.MVszW.strings[j].lppszW : "[NULL]");
-				}
-				break;
-			case PT_MV_SYSTIME:
-				g_print (" PT_MV_SYSTIME");
-				break;
-			case PT_MV_CLSID:
-				g_print (" PT_MV_CLSID");
-				break;
-			case PT_MV_BINARY:
-				g_print (" PT_MV_BINARY (%d items)", lpProp->value.MVbin.cValues);
-				for (j = 0; j < lpProp->value.MVbin.cValues; j++) {
-					g_print ("\n   item[%d] (size %d)\n", j, lpProp->value.MVbin.bin[j].cb);
-					dump_bin (lpProp->value.MVbin.bin[j].lpb, lpProp->value.MVbin.bin[j].cb, "     ");
-				}
-				g_print ("\n---");
-				break;
-			default:
-				g_print (" - Unknown type 0x%04X", lpProp->ulPropTag & 0xFFFF);
-			}
-		}
-	}
-	g_print ("\n");
-}
-
-/* Attention: Devs at work;-) */
-
 static void
 exchange_mapi_util_bin_append_uint16 (TALLOC_CTX *mem_ctx, struct Binary_r *bin, const uint16_t val)
 {
@@ -1018,11 +823,92 @@ exchange_crlf_to_lf (const gchar *in)
 	return out;
 }
 
+/**
+ * exchange_mapi_util_profiledata_from_settings:
+ * @empd: destination for profile settings
+ * @settings: a #CamelMapiSettings
+ *
+ * Sets the members of an ExchangeMapiProfileData instance to
+ * reflect the account settings in @settings.
+ *
+ * @note: no allocation is done, so do not finalize @settings and the
+ *        respective underlying pointers until you no longer need the
+ *        profile data.
+ **/
+void
+exchange_mapi_util_profiledata_from_settings (ExchangeMapiProfileData *empd, CamelMapiSettings *settings)
+{
+	CamelNetworkSettings *network_settings;
+	CamelNetworkSecurityMethod security_method;
+
+	network_settings = CAMEL_NETWORK_SETTINGS (settings);
+	security_method = camel_network_settings_get_security_method (network_settings);
+
+	empd->use_ssl = (security_method != CAMEL_NETWORK_SECURITY_METHOD_NONE);
+	empd->domain = camel_mapi_settings_get_domain (settings);
+	empd->krb_sso = camel_mapi_settings_get_kerberos (settings);
+	empd->krb_realm = camel_mapi_settings_get_realm (settings);
+}
+
+gboolean
+exchange_mapi_util_trigger_krb_auth (const ExchangeMapiProfileData *empd, GError **error) {
+	gint success = FALSE;
+	GError *local_error = NULL;
+	GDBusConnection *connection;
+	GDBusMessage *message, *reply;
+	gchar *name;
+
+	connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, &local_error);
+	if (local_error) {
+		g_warning ("could not get system bus: %s\n",
+			   local_error->message);
+		g_propagate_error (error, local_error);
+		return FALSE;
+	}
+
+	g_dbus_connection_set_exit_on_close (connection, FALSE);
+	/* Create a new message on the KRB_DBUS_INTERFACE */
+	message = g_dbus_message_new_method_call (KRB_DBUS_INTERFACE,
+						  KRB_DBUS_PATH,
+						  KRB_DBUS_INTERFACE,
+						  "acquireTgt");
+	if (!message) {
+		g_object_unref (connection);
+		return FALSE;
+	}
+
+	/* Appends the data as an argument to the message */
+	name = g_strdup_printf ("%s@%s", empd->username, empd->krb_realm);
+	g_dbus_message_set_body (message, g_variant_new ("(s)", name));
+
+	/* Sends the message: Have a 300 sec wait timeout  */
+	reply = g_dbus_connection_send_message_with_reply_sync (connection, message, G_DBUS_SEND_MESSAGE_FLAGS_NONE, 300 * 1000, NULL, NULL, &local_error);
+	g_free (name);
+
+	if (local_error) {
+		g_warning ("%s: %s\n", G_STRFUNC, local_error->message);
+		g_propagate_error (error, local_error);
+	}
+
+	if (reply) {
+		GVariant *body = g_dbus_message_get_body (reply);
+		if (body) {
+			g_variant_get (body, "(b)", &success);
+		}
+		g_object_unref (reply);
+	}
+
+	/* Free the message */
+	g_object_unref (message);
+	g_object_unref (connection);
+
+	return success && !local_error;
+}
+
+
 /**
  * exchange_mapi_util_profile_name:
- * @username: User name of the profile
- * @domain: Domain name of the profile
- * @hostname: Server host name
+ * @empd: profile information used to construct the name
  * @migrate: whether migrate old profile name to a new one
  *
  * Constructs profile name from given parameters and
@@ -1030,18 +916,20 @@ exchange_crlf_to_lf (const gchar *in)
  * rename old profile name string to a new name, if requested.
  **/
 gchar *
-exchange_mapi_util_profile_name (const gchar *username, const gchar *domain, const gchar *hostname, gboolean migrate)
+exchange_mapi_util_profile_name (const ExchangeMapiProfileData *empd, gboolean migrate)
 {
 	gchar *res;
 
-	res = g_strdup_printf ("%s@%s@%s", username, domain, hostname);
+	res = g_strdup_printf ("%s@%s@%s", empd->username, empd->domain,
+			       empd->server);
 	res = g_strcanon (res, "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789@.-", '_');
 
 	if (migrate) {
 		/* expects MAPIInitialize already called! */
 		gchar *old_name;
 
-		old_name = g_strdup_printf ("%s@%s", username, domain);
+		old_name = g_strdup_printf ("%s@%s", empd->username,
+					    empd->domain);
 		old_name = g_strcanon (old_name, "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789@", '_');
 
 		exchange_mapi_rename_profile (old_name, res);
@@ -1107,25 +995,14 @@ exchange_mapi_utils_add_spropvalue (TALLOC_CTX *mem_ctx, struct SPropValue **val
 	g_return_val_if_fail (values_array != NULL, FALSE);
 	g_return_val_if_fail (n_values != NULL, FALSE);
 
-	if ((prop_tag & 0xFFFF) == PT_DOUBLE) {
-		uint64_t zero = 0;
-
-		/* add an empty fake value and rewrite it */
-		*values_array = add_SPropValue (mem_ctx, *values_array, n_values, PROP_TAG(PT_LONG, 0x0001), &zero);
-
-		((*values_array)[(*n_values) - 1]).ulPropTag = prop_tag;
-		((*values_array)[(*n_values) - 1]).dwAlignPad = 0;
-		memcpy (&((*values_array)[(*n_values) - 1]).value.dbl, prop_value, 8);
-	} else {
-		*values_array = add_SPropValue (mem_ctx, *values_array, n_values, prop_tag, prop_value);
-	}
+	*values_array = add_SPropValue (mem_ctx, *values_array, n_values, prop_tag, prop_value);
 
 	return TRUE;
 }
 
 /* similar as exchange_mapi_utils_add_spropvalue, just here is not used prop_tag, but named id */
 gboolean
-exchange_mapi_utils_add_spropvalue_named_id (ExchangeMapiConnection *conn, mapi_id_t fid, TALLOC_CTX *mem_ctx, struct SPropValue **values_array, uint32_t *n_values, uint32_t named_id, gconstpointer prop_value)
+exchange_mapi_utils_add_spropvalue_namedid (ExchangeMapiConnection *conn, mapi_id_t fid, TALLOC_CTX *mem_ctx, struct SPropValue **values_array, uint32_t *n_values, uint32_t named_id, gconstpointer prop_value)
 {
 	uint32_t prop_tag;
 
